;;; tinydebian.el --- Linux (Debian) utilities.
;; $Id: tinydebian.el,v 1.21 2005/02/16 16:22:10 jaalto Exp $

;;{{{ Id

;; Copyright (C)    2001-2005 Jari Aalto
;; Keywords:        extensions
;; Author:          Jari Aalto
;; Maintainer:      Jari Aalto
;;
;; To get information on this program, call M-x tinydebian-version.
;; Look at the code with folding.el

;; COPYRIGHT NOTICE
;;
;; This program is free software; you can redistribute it and/or modify it
;; under the terms of the GNU General Public License as published by the Free
;; Software Foundation; either version 2 of the License, or (at your option)
;; any later version.
;;
;; This program is distributed in the hope that it will be useful, but
;; WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
;; or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
;; for more details.
;;
;; You should have received a copy of the GNU General Public License along
;; with GNU Emacs.  If you did not, write to the Free Software Foundation,
;; Inc., 675 Mass Ave., Cambridge, MA 02139, USA.

;;}}}
;;{{{ Install

;; ........................................................ &t-install ...
;;   Put this file on your Emacs-Lisp load path, add following into your
;;   $HOME/.emacs startup file
;;
;;      (setq tinydebian-:load-hook 'tinydebian-install)
;;      (require 'tinydebian)
;;
;;   If you have any questions or bug report to send, use this function:
;;
;;      M-x tinydebian-submit-bug-report       send question, feedback,bug report
;;
;;  To read the documentation after file has been loaded, call
;;
;;      M-x tinydebian-version

;;}}}
;;{{{ Documentation

;; ..................................................... &t-commentary ...

;;; Commentary:

;;  Overview of features
;;
;;      This package contains utilities for the Debian System Administarator,
;;      to help administring Debian in daily tasks and submitting bug
;;      reports from Emacs. Learn more about debian at
;;      http://www.debian.org/
;;
;;      o   colorize /var/log files like messages, syslog etc.
;;      o   Report Debian bug with M-x ... #todo
;;
;;  Quick start

;;}}}

;;; Change Log:

;;; Code:

;;{{{ setup: libraries

(require 'tinylibm)

(eval-when-compile (ti::package-use-dynamic-compilation))


(eval-and-compile
  ;;  Forward declarations to shut up byte compiler.
  (defvar font-lock-mode)
  (defvar font-lock-keyword-face)
  (defvar global-font-lock-mode)
  (defvar font-lock-keywords)
  (defvar font-lock-defaults))


(ti::package-defgroup-tiny TinyDebian tinydebian-: extensions
  "Debian System administrator's grabbag of utilities.")

;;}}}
;;{{{ setup: hooks

;;; ......................................................... &v-hooks ...


(defcustom tinydebian-:load-hook nil
  "*Hook run when file has been loaded."
  :type  'hook
  :group 'TinyDebian)


;;}}}
;;{{{ setup: user config

;;; ................................................... &v-user-config ...

(defcustom tinydebian-:dummy nil
  "*"
  :type  'string
  :group 'TinyDebian)


(defcustom tinydebian-:buffer-tiger "*tinydebian-tiger*"
  "*Buffer name where to generate tiger(1) mail report chmod fixes.
See function `tinydebian-audit-report-tiger'."
  :type  'string
  :group 'TinyDebian)


(defface tinydebian-:warn-face
  '((((class color) (background light))
     (:background "green"))
    (((class color) (background dark))
     (:background "sea green"))
    (((class grayscale monochrome)
      (background light))
     (:background "black"))
    (((class grayscale monochrome)
      (background dark))
     (:background "white")))
  "Face used for warnings."
  :group 'TinyDebian)

;;; Color loading section  This is messy *Blech!*
;;
(defface tinydebian-:item-face
  '((((class color) (background light))
     (:foreground "green4"))
    (((class color) (background dark))
     (:foreground "green3")))
  "Face used for noticing important items."
  :group 'TinyDebian)


;;}}}
;;{{{ setup: -- private

;;; ....................................................... &v-private ...


(defvar tinydebian-:font-lock-keywords-adaptive-date nil
  "Flag to signal that current time is used to display today's log.
For exmple in /etc/syslog today's log log entries are highlighted
differently that previous days. However this must be changed in
next day, because the daye changes.

This flags says, that adaptive-date regexs must be used.")

(make-variable-buffer-local 'tinydebian-:font-lock-keywords-adaptive-date)


(defvar tinydebian-:bin-dpkg (executable-find "dpkg")
  "*Location of `dpkg' binary.")

(defvar tinydebian-:severity-list
  '(("critical"
     "Makes unrelated software on the system (or the whole system) break,
or causes serious data loss, or introduces a security hole on systems where
you install the package.")
    ("grave"
     "Makes the package in question unuseable or mostly so, or causes data
loss, or introduces a security hole allowing access to the accounts of users
who use the package.")
    ("serious"
     "Severe violation of Debian policy (that is, it violates a
\"must\" or \"required\" directive), or, in the package maintainer's
opinion, makes the package unsuitable for release.")
    ("important"
     "A bug which has a major effect on the usability of a package,
without rendering it completely unusable to everyone.")
    ("normal"
     "The default value, applicable to most bugs.")
    ("minor"
     "A problem which doesn't affect the package's usefulness, and is
presumably trivial to fix.")
    ("wishlist"
     "For any feature request, and also for any bugs that are very
difficult to fix due to major design considerations.")
    ("fixed"
     "For bugs that are fixed but should not yet be closed. This is an
exception for bugs fixed by non-maintainer uploads. Note: the "fixed"
tag should be used instead."))
  "The bug system records a severity level with each bug report.
This is set to normal by default, but can be overridden either by supplying a Severity line in the pseudo-header when the bug is submitted Severity or error.
http://www.debian.org/Bugs/Developer#severities")


(defvar tinydebian-:severity-selected nil
  "Function `tinydebian-severity-select-*' sets this to user selection.")

(defconst tinydebian-:menu-severity
  '("\
Severity: ?h)elp c)rit g)rave s)erious i)import RET-n)orm m)inor w)ish f)ixed"
    ;; NOTE: These function are automatically created, you don't find
    ;; then with C-s. See `tinydebian-install-severity-functions'
    ((?c .      ( (call-interactively 'tinydebian-severity-select-critical)))
     (?g .      ( (call-interactively 'tinydebian-severity-select-grave)))
     (?s .      ( (call-interactively 'tinydebian-severity-select-serious)))
     (?i .      ( (call-interactively 'tinydebian-severity-select-important)))
     (?n .      ( (call-interactively 'tinydebian-severity-select-normal)))
     (?\C-m .   ( (call-interactively 'tinydebian-severity-select-normal)))
     (?m .      ( (call-interactively 'tinydebian-severity-select-minor)))
     (?w .      ( (call-interactively 'tinydebian-severity-select-wishlist)))
     (?f .      ( (call-interactively 'tinydebian-severity-select-fixed)))))
  "Severity menu.

The bug system records a severity level with each bug report. This is set
to normal by default, but can be overridden either by supplying a Severity
line in the pseudo-header when the bug is submitted (see the instructions
for reporting bugs), or by using the severity command with the control
request server.

critical
    makes unrelated software on the system (or the whole system) break, or
    causes serious data loss, or introduces a security hole on systems where
    you install the package.

grave
    makes the package in question unuseable or mostly so, or causes data loss,
    or introduces a security hole allowing access to the accounts of users who
    use the package.

serious
    is a severe violation of Debian policy (that is, it violates a \"must\" or
    \"required\" directive), or, in the package maintainer's opinion, makes the
    package unsuitable for release.

important
    a bug which has a major effect on the usability of a package, without
    rendering it completely unusable to everyone.

normal
    the default value, applicable to most bugs.

minor
    a problem which doesn't affect the package's usefulness, and is presumably
    trivial to fix.

wishlist
    for any feature request, and also for any bugs that are very difficult to
    fix due to major design considerations.

fixed
    for bugs that are fixed but should not yet be closed. This is an exception
    for bugs fixed by non-maintainer uploads. Note: the \"fixed\" tag should be
    used instead.  Certain severities are considered release-critical, meaning
    the bug will have an impact on releasing the package with the stable
    release of Debian. Currently, these are critical, grave and serious.")


(defvar tinydebian-:tags-list
  '(("patch"
     "A patch or some other easy procedure for fixing the bug is included
in the bug logs. If there's a patch, but it doesn't resolve the bug
adequately or causes some other problems, this tag should not be used.")
    ("wontfix"
     "This bug won't be fixed. Possibly because this is a choice between
two arbitrary ways of doing things and the maintainer and submitter prefer
different ways of doing things, possibly because changing the behaviour
will cause other, worse, problems for others, or possibly for other reasons.")
    ("moreinfo"
     "This bug can't be addressed until more information is provided by
the submitter. The bug will be closed if the submitter doesn't provide
more information in a reasonable (few months) timeframe. This is for
bugs like "It doesn't work". What doesn't work?.")
    ("unreproducible"
     "This bug can't be reproduced on the maintainer's system.
Assistance from third parties is needed in diagnosing the cause of the problem.")
    ("help"
     "The maintainer is requesting help with dealing with this bug.")
    ("pending"
     "The problem described in the bug is being actively worked on,
i.e. a solution is pending.")
    ("fixed"
     "This bug is fixed or worked around (by a non-maintainer upload,
for example), but there's still an issue that needs to be resolved.
This tag replaces the old \"fixed\" severity.")
    ("security"
     "This bug describes a security problem in a package (e.g., bad
permissions allowing access to data that shouldn't be accessible;
buffer overruns allowing people to control a system in ways they
shouldn't be able to; denial of service attacks that should be fixed, etc).
Most security bugs should also be set at critical or grave severity.")
    ("upstream"
     "This bug applies to the upstream part of the package.")
    ("potato"
     "This bug particularly applies to the potato release of Debian.")
    ("woody"
     "This bug particularly applies to the (unreleased) woody distribution.")
    ("sid"
     "This bug particularly applies to an architecture that is
currently unreleased (that is, in the sid distribution)."))
  "Each bug can have zero or more of a set of given tags.
These tags are displayed in the list of bugs when you look at a
package's page, and when you look at the full bug log.
http://www.debian.org/Bugs/Developer")

(defvar tinydebian-:wnpp-buffer "*TinyDebian WNPP*"
  "WNPP question buffer.")


(defvar tinydebian-:menu-wnpp-selected nil
  "Placeholder of selection from `tinydebian-:menu-wnpp'.")

(defconst tinydebian-:menu-wnpp
  (list
   '(format
    "TinyDebian:WNPP%s 1p)package 2o)rphan 3a)dopt 4n)ew package ?)help q)uit"
    (if tinydebian-:menu-wnpp-selected
        (format ";%s " (symbol-name tinydebian-:menu-wnpp-selected))
      ""))
   (list
    '(?1 . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?p . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?P . ( (setq tinydebian-:menu-wnpp-selected 'package)))
    '(?2 . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?o . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?O . ( (setq tinydebian-:menu-wnpp-selected 'oprhan)))
    '(?3 . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?a . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?A . ( (setq tinydebian-:menu-wnpp-selected 'adopt)))
    '(?4 . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    '(?n . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    '(?N . ( (setq tinydebian-:menu-wnpp-selected 'new)))
    ))
  ;;  This message is straight from reportbug(1)
  ;;  'apt-get install reportbug'
  "What request type? If none of these things mean anything to you, or
you are trying to report a bug in an existing package)

1 p    ITP, `Intent To Package'. Please submit a package description
       along with copyright and URL in such a report.

2 o    The package has been `Orphaned'. It needs a new maintainer as soon as
       possible.

3 a    RFA, this is a `Request for Adoption'. Due to lack of time, resources,
       interest or something similar, the current maintainer is asking for
       someone else to maintain this package. He/she will maintain it in the
       meantime, but perhaps not in the best possible way. In short: the
       package needs a new maintainer.

4 n    RFP, this is a `Request For Package'. You have found an interesting piece of
       software and would like someone else to maintain it for Debian. Please
       submit a package description along with copyright and URL in such a
       report.

q      Quit menu.
")



;;}}}
;;{{{ setup: -- version

;;; ....................................................... &v-version ...

;;;###autoload (autoload 'tinydebian-version "tinydebian" "Display commentary." t)
(eval-and-compile
(ti::macrof-version-bug-report
 "tinydebian.el"
 "tinydebian"
 tinydebian-:version-id
 "$Id: tinydebian.el,v 1.21 2005/02/16 16:22:10 jaalto Exp $"
 '(tinydebian-version-id
   tinydebian-:load-hook
   tinydebian-:font-lock-keywords-adaptive-date
   tinydebian-:bin-dpkg
   tinydebian-:severity-list
   tinydebian-:severity-selected
   tinydebian-:tags-list)))


;;}}}

;;; ########################################################### &Funcs ###

;;{{{ Install: bindings

;;; ........................................................ &bindings ...

;; #todo:
(defun tinydebian-default-bindings ()
  "Define default key bindings to `tinydebian-mode-map'.")

;;}}}
;;{{{ Install: generate severity function etc.

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-install-severity-functions ()
  "Generate `tinydebian-severity-select-*' user functions."
  ;; Generate functions on run-time.
  (mapcar
   (function
    (lambda (x)
      (let ((sym (intern (format "tinydebian-severity-select-%s"  x)))
            def)
        (setq def
              (` (defun (, sym) ()
                   "Set Severity level `tinydebian-:severity-selected'."
                   (interactive)
                   (setq  tinydebian-:severity-selected (, x)))))
        (eval def))))
   '("critical"
     "grave"
     "serious"
     "important"
     "normal"
     "minor"
     "wishlist"
     "fixed")))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-find-file-hooks ()
  "Install `font-lock-keywords' for log files."
  (tinydebian-font-lock-keywords))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-install-font-lock-keywords (&optional uninstall)
  "Install colors to all current buffers."
  (dolist (buffer (buffer-list))
    (with-current-buffer buffer
      (tinydebian-font-lock-keywords uninstall))))

;;; ----------------------------------------------------------------------
;;;
;;;###autoload (defalias 'tinydebian-bug-report-mail 'debian-reportbug-package)
;;;###autoload
(defun tinydebian-install (&optional uninstall)
  "Install or UNINSTALL package."
  (interactive "P")

  ;;  This just hides from byte compiler fucntion definition
  ;;  so that it does not remember how amny arguments it takes
  ;;
  ;;  function tinydebian-bug-report-mail used to take 0+ arguments, now takes 1
  ;;  function tinydebian-bug-report-mail defined multiple times in this file
  ;;
  (let ((sym 'tinydebian-bug-report-mail))
    (defalias sym 'debian-reportbug-package))

  (cond
   (uninstall
    ;;(remove-hook 'write-file-hooks 'tinydebian-auto-save)
    (tinydebian-install-font-lock-keywords 'uninstall)
    (remove-hook 'find-file-hooks 'tinydebian-find-file-hooks)
    nil)
   (t
    ;; (add-hook 'write-file-hooks 'tinydebian-auto-save)
    (tinydebian-install-font-lock-keywords)
    (add-hook 'find-file-hooks  'tinydebian-find-file-hooks)
    nil)))


;;}}}
;;{{{ Utility functions

;;; ----------------------------------------------------------------------
;;;
(defsubst tinydebian-string-delete-newlines (string)
  "Delete newlines from STRING."
  (ti::string-regexp-delete "[\r\n]" string))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-font-lock-keywords (&optional uninstall)
  "Add color support to various log files by setting
`font-lock-keywords'."
  (interactive)
  (let* ((today  (ti::date-standard-rfc-regexp "mon-date"))
         ;; (cs     (or comment-start-skip "[ \t]+"))
         (file   "")
         keywords)

    (when (stringp buffer-file-name)
      (setq file (or buffer-file-name "no-name?")))

    (setq
     keywords
     (cond

      ;; ............................................. Linux log files ...
      ;; /var/log/

      ((string-match "/log/messages$" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "restarted\\|started"
                       "\\|ignoring"
                       "\\|Linux version.*")
               0 'font-lock-comment-face))))

      ((string-match "mail\\.log\\|mail\\.info" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ ++[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              '("timed out\\|did not.*"
                0 tinydebian-:warn-face)
              (list
               (concat "\\(from\\|to\\)=\\([^ ,\t\r\n]+\\)")
               2 'font-lock-comment-face))))


      ((string-match "daemon\\.log" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "connection attempt" ;;  See "iplogger" package
                       )
               0 'tinydebian-:warn-face)
              (list
               (concat "signal +[0-9]+\\|no such user"
                       "\\|connect from .*")
               0 'font-lock-comment-face))))


      ((string-match "auth\\.log" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "opened +for +[^ \t\r\n]+")
               0 'tinydebian-:warn-face)
              '( "for user \\(root\\)"
                 1 font-lock-string-face)
              '( "from \\([^ \t\r\n]+\\)"
                 1 font-lock-type-face)
              '( "for +\\([^ \t\r\n]+\\) +from"
                 1 font-lock-comment-face)
              '( "for user +\\([^ \t\r\n]+\\)"
                 1 font-lock-comment-face))))

      ((string-match "syslog" file)
       ;; font-lock-constant-face
       (make-local-variable 'font-lock-defaults)
       (setq font-lock-keywords
             (list
              (list (concat today " +[0-9]+:+[0-9]+:+[0-9]+")
                    0 'font-lock-function-name-face)
              (list
               (concat
                "^... +[0-9]+ +[0-9]+:+[0-9]+:+[0-9]+")
               0 'font-lock-reference-face)
              (list
               (concat "Invalid.*"
                       ;; portmap[135]: cannot bind udp: Address already in use
                       "\\|cannot"
                       "\\|Connection timed out"
                       ;;  See iplogger(1)
                       "\\|connection attempt"
                       ;;  See portsentry(1)
                       "\\|attackalert:.* +to +.*port.*"
                       ;;  apm -s failed
                       "\\| failed"
                       "\\|did not .*")
               0 'tinydebian-:warn-face)
              '("to=\\([^ \t\r\n]+\\)"
                1 font-lock-comment-face)
              '("(\\([^ )\t\r\n]+\\)) CMD "
                1 font-lock-comment-face)
              '("CMD .*"
                0 font-lock-constant-face)
              '("inetd"2
                0 font-lock-type-face)
              (list
               (concat
                "program exit.*\\|.*started.*"
                ;;  btpd daemon
                "\\|synchronisation lost"
                )
                0 font-lock-keyword-face))))

      ))
    (when keywords
      (cond
       (uninstall
         (setq font-lock-keywords nil))
       ((or font-lock-mode
            global-font-lock-mode
            (font-lock-mode-maybe 1))
        (setq font-lock-keywords keywords))))))


;;}}}
;;{{{ Dpkg, apt functions

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-read-field-content-1 ()
  "Read content. Point must be positionioned at Field:-!-."
  (let* ((str (if (looking-at " +\\(.*\\)")
                  (match-string 1))))
    (while (and (not (eobp))
                (zerop (forward-line 1))  ;; Did it
                (looking-at "^\\( +.*\\)"))
      (setq str (concat (or str "") (match-string 1))))
    str))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-read-field-content (&optional field)
  "Read FIELD forward. FIELD ust be name like `Package'.
Be sure to call `tinydebian-package-narrow-to-region' first."
  (when (re-search-forward (format "^%s:" field) nil t)
    (tinydebian-package-read-field-content-1)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-parse-info-all ()
  "Parse all fields forward. Return '((field . info) (field . info) ..)."
  (let* (field
         alist)
    (while (re-search-forward "^\\([^ \t\r\n]+\\):" nil t)
      (setq field (match-string 1))
      (push (cons field (tinydebian-package-read-field-content-1))
            alist))
    (nreverse alist)))

;;; ----------------------------------------------------------------------
;;;
(defmacro tinydebian-package-narrow-to-region (&rest body)
  "Search dpkg -s result from current point forward and narrow around it.
Point is put at the beginning of region.
Variable `package' contains the package name."
  (`
   (let* (beg-narrow
          package)
     (when (re-search-forward "^Package: +\\([^ \t\r\n]+\\) *$" nil t)
       (setq beg-narrow (line-beginning-position))
       (setq package (match-string 1))
       (when (re-search-forward "^[ \t]*$" nil t)
         (ti::narrow-safe beg-narrow (point)
           (ti::pmin)
           (,@ body)))))))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-parse-depends-1 ()
  "Parse `Depends' field content from current point forward.
There must nothing else in the buffer."
  (let* (name
         op
         ver
         list)
    (while (re-search-forward "\\([a-z][^ ,()\t\r\n]+\\)" nil t)
      (setq name (ti::remove-properties (match-string 1))
            op   nil
            ver  nil)
      (cond
       ((looking-at " +(\\([=><]+\\) +\\([^ ,()\t\r\n]+\\))")
        (setq op   (ti::remove-properties (match-string 1))
              ver  (ti::remove-properties (match-string 2))))
       ((looking-at " *,?")))
      (goto-char (match-end 0))
      (push (list name op ver) list))
    (nreverse list)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-parse-depends (depends)
  "Parse `Depends' field from DEPENDS string.
Example of the DEPENDS string:

    \"libc6 (>= 2.2.4-2), cron (>= 3.0pl1-42)\"

Returned list is

   '((\"libc6\" \">=\" \"2.2.4-2\")
     (\"cron\"  \">=\" \"3.0pl1-42\"))."
   (with-temp-buffer
      (insert depends)
      (ti::pmin)
      (tinydebian-package-status-parse-depends-1)))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-call-process (binary buffer args)
  "Call BINARY with list of ARGS and print output to current buffer or BUFFER."
  (apply 'call-process
         binary
         nil
         (or buffer (current-buffer))
         nil
         args))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-apt-file (package)
  "Use apt-file PACKAGE (must be installed separately) to find upstream."
  (let* ((bin (executable-find "apt-file")))
    (cond
     ((null bin)
      (message "TinyDebian: no `apt-fil' found along PATH (emacs `exec-path').")
      (message "TinyDebian: Please run 'apt-get install apt-file'")
      nil)
     (t
      ;; #todo:
      ))))


;;; ----------------------------------------------------------------------
;;;
;;; Package: autolog
;;; Status: install ok installed
;;; Priority: extra
;;; Section: admin
;;; Installed-Size: 45
;;; Maintainer: Nicols Lichtmaier <nick@debian.org>
;;; Version: 0.35-10
;;; Depends: libc6 (>= 2.2.4-2), cron (>= 3.0pl1-42)
;;; Recommends: mail-transport-agent
;;; Conffiles:
;;;  /etc/autolog.conf a3fcae584ed74543a4a943e722593ff6
;;;  /etc/cron.d/autolog 805d268ea44c645299defc1c14495282
;;; Description: Terminates connections for idle users
;;;  Autolog terminates connections considered to be idle based on a large
;;;  variety of parameters.
;;;
(defun tinydebian-package-status-dpkg-s (package)
  "Consult dpkg -s PACKAGE"
  (let* ((dpkg  tinydebian-:bin-dpkg))
    (cond
     ((not dpkg)
      (message "TinyDebian: no `dpkg' found along PATH (emacs `exec-path').")
      nil)
     (t
      (with-temp-buffer
        (message "TinyDebian: Running ... dpkg -s %s" package)
        (tinydebian-call-process dpkg nil (list "-s" package))
        (ti::pmin)
        (when (re-search-forward "^Use dpkg" nil t)
          (message "TinyDebian: `dpkg`-s %s' returned error [%s]"
                   package
                   (buffer-string)))
        (tinydebian-package-parse-info-all))))))


;;; ----------------------------------------------------------------------
;;; dpkg -S dh_make
;;;
;;; debhelper: /usr/bin/dh_makeshlibs
;;; dh-make: /usr/share/debhelper/dh_make/debian/postrm.ex
;;; dh-make: /usr/share/debhelper/dh_make/native
;;; dh-make: /usr/share/debhelper/dh_make/debian/changelog
;;; dh-make: /usr/share/debhelper/dh_make/debianl/shlibs.local.ex
;;; dh-make: /usr/share/man/man1/dh_make.1.gz
;;; dh-make: /usr/bin/dh_make
;;; dh-make: /usr/share/debhelper/dh_make/debiank/README.Debian
;;; dh-make: /usr/share/debhelper/dh_make/debianm/control
;;; dh-make: /usr/share/debhelper/dh_make/debian/init.d.ex
;;; dh-make: /usr/share/debhelper/dh_make/debian/cron.d.ex
;;; dh-make: /usr/share/debhelper/dh_make/debianm/rules
;;; dh-make: /usr/share/debhelper/dh_make/licenses/lgpl
;;; dh-make: /usr/share/debhelper/dh_make/debiank/control
;;; dh-make: /usr/share/debhelper/dh_make/debians/rules
;;; dh-make: /usr/share/debhelper/dh_make/debianl/package1.dirs
;;; dh-make: /usr/share/debhelper/dh_make/native/changelog
;;; dh-make: /usr/share/debhelper/dh_make/licenses/bsd
;;; dh-make: /usr/share/debhelper/dh_make/debianm/package-doc.files
;;; dh-make: /usr/share/debhelper/dh_make/debians/watch.ex
;;; dh-make: /usr/share/debhelper/dh_make/licenses/gpl
;;; dh-make: /usr/share/debhelper/dh_make/licenses/blank
;;;
(defun tinydebian-package-status-dpkg-S-parse (package)
  "Examine dpkg -S PACKAGE listing and return upstream package name."
  (ti::pmin)
  (when (re-search-forward (concat ":.*/\\([a-z]+\\)/"
                                   package
                                   "[/ \t\r\r]"
                                   )
                           nil t)
    (match-string 1)))


;;; ----------------------------------------------------------------------
;;;
;;;
(defun tinydebian-package-status-dpkg-S (file)
  "Consult dpkg -S FILE
In this case, the package is unknown and the upstream package is
searched. If upstream package is found, call `tinydebian-package-status-dpkg-s'. "
  (let* ((dpkg  tinydebian-:bin-dpkg))
    (cond
     ((not dpkg)
      (message "TinyDebian: no `dpkg' found along PATH (emacs `exec-path').")
      nil)
     (t
      (with-temp-buffer
        (message "TinyDebian: Running ... dpkg -S %s" file)
        (tinydebian-call-process dpkg nil (list "-S" file))
        (let ((upstream (tinydebian-package-status-dpkg-S-parse file)))
          (cond
           ((null upstream)
            (message "TinyDebian: dpkg -S doesn't know upstream package for `%s'"
                     file)
            nil)
           (t
            (tinydebian-package-status-dpkg-s upstream)))))))))



;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-wnpp-main-interactive ()
  "Ask the type of request for WNPP package.
References:
  `tinydebian-:menu-wnpp'
  `tinydebian-:menu-wnpp-selected'"
  (setq tinydebian-:menu-wnpp-selected nil)
  (ti::menu-menu 'tinydebian-:menu-wnpp)
  tinydebian-:menu-wnpp-selected)

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-buffer-ask-input (message buffer &optional clear)
  "Write MESSAGE to the buffer ans ask user to type input.
The MESSAGE should contgain properly formatted text."
  (let* ((buffer (ti::temp-buffer buffer clear)))
    ;; (switch-to-buffer buffer)
    ;; #todo:
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-wnpp-main (request-type)
  "Submit REQUEST-TYPE against WNPP pseudo package.
WNPP is used for requesting to be a new Debian maintainer and
for taking maintenance of other packages. Refer to
http://www.debian.org/devel/wnpp and
http://www.debian.org/doc/packaging-manuals/developers-reference/ch-pkgs.en.html
and topic \"5.1 New Packages\"

REQUEST-TYPE can be symbol:

  'package 'orphan 'adopt or 'new.
  See http://www.debian.org/devel/wnpp for more information

References:

  `tinydebian-:menu-wnpp'."
  (interactive (list (tinydebian-package-wnpp-main-interactive)))
  (cond
   ((eq request-type 'package)
    )
   ((eq request-type 'new)
    (let ((name  (read-string "Proposed package name: ")))
      ;;  Checking status database

      nil))
   ((eq request-type 'orphan)
    )
   ((eq request-type 'adopt)
    )
   ((eq request-type 'new)
    )
   (t
    ;;  Nothing to do
    nil)))




;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-status-main (package)
  "Find out PACKAGE details."
  (or (tinydebian-package-status-dpkg-s package)
      (tinydebian-package-status-dpkg-S package)
      (tinydebian-package-status-apt-file package)
      (if (string-match "^wnpp" package)
          (error "\
TinyDebian: package WNPP is special. Use tinydebian-package-wnpp-main instead."))
      (error "Tinydebian: Can't find package information. `%s'" package)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-info-from-buffer (buffer)
  "Parse dpkg -s from BUFFER. Tge buffer must contain nothing else."
  (with-current-buffer buffer
    (ti::pmin)
    (tinydebian-package-parse-info-all)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-info-from-user (package &optional prompt)
  "Ask user to supply `dpkg -s PACKAGE' to *scratch* and parse it."
  (let* ((buffer "*scratch*"))
    (read-string
     (or prompt
         (format
          (concat "[TinyDebian] insert `dpkg -s %s' to "
                  buffer
                  " and press RET when ready: ")
          package)))
    (display-buffer (or (get-buffer buffer)
                        (get-buffer-create buffer)))
    (tinydebian-package-info-from-buffer buffer)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-package-info (&optional package prompt)
  "Get PACKAGE information. See`tinydebian-package-status'.
If PACKAGE is nil and `tinydebian-:bin-dpkg' is not available,
ask with PROMPT."
    (let* ((dpkg  tinydebian-:bin-dpkg))
      (or package
          (setq package (read-string
                         (or prompt
                             "[TinyDebian] Package name: "))))
      (or (and dpkg
               (tinydebian-package-status-main package))
          (tinydebian-package-info-from-user package prompt))))


;;}}}
;;{{{ Bug reporting interface

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-general ()
  "Return relevant system information."
  (interactive)
  (let* (
         )
    "" ;; #todo:
    ))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-depends (info &optional depend-key)
  "Return additional Dependency INFO from item `Depends'.
DEPEND-KEY can be \"Depends\" or \"Pre-Depends\".

Example:

  Versions of packages autolog depends on:
  ii  cron            3.0pl1-72  management of regular background p
  ii  libc6           2.2.5-3    GNU C Library: Shared libraries an."
  (let* ((depends (cdr-safe (assoc
                             (or depend-key "Depends")
                             info)))
         str)
    (when depends
      (setq str "")
      (dolist (dep-info
               (tinydebian-package-status-parse-depends depends))
        (multiple-value-bind (package op version)
            dep-info

          ;; Not used yet, quiet byte compiler
          (if op
              (setq op op))
          (if version
              (setq version version))

          (let* (info2
                 desc
                 ver)
            (setq info2
                  (tinydebian-package-info
                   package
                   (format "\
\[TinyDebian] Depend. Insert `dpkg -s %s' to *scratch* and press RET: "
                           package)))
            (setq ver  (cdr-safe (assoc "Version" info2)))
            ;; cut first few characters
            (when (setq desc (cdr-safe (assoc "Description" info2)))
              (setq desc (ti::string-left desc 45)))
            (setq str
                  (concat
                   str
                   (format "%-15s %-15s %s\n" package ver desc)))))))
    str))


;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os-architecture ()
  "Read architecture."
  (if (not tinydebian-:bin-dpkg)
      ""
    (with-temp-buffer
    (tinydebian-call-process
     tinydebian-:bin-dpkg  nil '("--print-installation-architecture"))
    (tinydebian-string-delete-newlines
     (buffer-string)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os-version ()
  "Read Debian version number."
  (let* ((file  "/etc/debian_version")
         (ret    (format "%s not found or readable." file)))
    (when (and (file-exists-p   file)
               (file-readable-p file))
      (with-temp-buffer
        (insert-file-contents-literally file)
        (setq ret
              (tinydebian-string-delete-newlines
               (buffer-string)))))
    ret))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-locale ()
  "Get locale information."
  (let* ((list
          '("LC_ALL"
            "LC_CTYPE"))
         val
         ret)
    (dolist (var list)
      (when (setq val (getenv var))
        (setq val (format "%s=%s" var val))
        (setq ret (if (null ret)
                      val
                    (concat ret ", " val)))))
    ret))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-system-info-os ()
  "Return OS information.
Debian Release: 3.0
Architecture: i386
Kernel: Linux terra 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US."
  (let* ((kernel       (tinydebian-string-delete-newlines
                        (ti::process-uname)))
         (architecture (tinydebian-bug-system-info-os-architecture))
         (release      (tinydebian-bug-system-info-os-version))
         (locale       (tinydebian-bug-system-info-locale)))
    (format "\
Debian Release: %s
Architecture: %s
Kernel: %s
Locale: %s"
            release
            architecture
            kernel
            locale)))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-severity ()
  "Select bug severity."
  (setq tinydebian-:severity-selected nil)
    (while (null tinydebian-:severity-selected)
      (ti::menu-menu 'tinydebian-:menu-severity)
      (unless tinydebian-:severity-selected
        (message "TinyDebian: Please select severity.")
        (sit-for 1)))
    tinydebian-:severity-selected)

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-bug-report-mail-insert-details (info)
  "Insert Details for apckage INFO into Mail."
  (ti::mail-text-start 'move)
  (insert "Package: " (cdr (assoc "Package" info)) "\n")
  (insert "Version: " (cdr (assoc "Version" info)) "\n")
  (insert "Severity: " (tinydebian-bug-severity)   "\n\n")
  (let* ((point       (point))
         (depends     (tinydebian-bug-system-info-depends info "Depends"))
         (pre-depends (tinydebian-bug-system-info-depends info "Pre-Depends")))
    (insert "\n\n-- System Information\n"
            (tinydebian-bug-system-info-os)
            (format "\n\n-- Versions of packages `%s depends on'.\n"
                    (cdr (assoc "Package" info)))
            (if pre-depends
                (concat "Pre-Depends:\n" pre-depends)
              "")
            (if depends
                (concat "Depends:\n" depends)))
    (goto-char point)))


;;; ----------------------------------------------------------------------
;;;
;;;###autoload
(defun tinydebian-bug-report-mail (info)
  "Submit Debian bug report. INFO is alist of attributes for package.
An example reportbug(1)' looks like

To: submit@bugs.debian.org
Subject: autolog ....
--text follows this line--
Package: autolog
Version: 0.35-10
Severity: wishlist



-- System Information
Debian Release: 3.0
Architecture: i386
Kernel: Linux foo 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US

Versions of packages autolog depends on:
ii  cron                          3.0pl1-72  management of regular background p
ii  libc6                         2.2.5-3    GNU C Library: Shared libraries an


Subject: autolog based on DNS and IP names
Package: autolog
Version: 0.35-10
Severity: wishlist



-- System Information
Debian Release: 3.0
Architecture: i386
Kernel: Linux terra 2.4.17 #1 Fri Feb 8 21:32:43 EET 2002 i686
Locale: LANG=en_US, LC_CTYPE=en_US

Versions of packages autolog depends on:
ii  cron                          3.0pl1-72  management of regular background p
ii  libc6                         2.2.5-3    GNU C Library: Shared libraries an."
  (interactive
   (progn
     (if (y-or-n-p "[TinyDebian] Submit bug report? ")
         (list (tinydebian-package-info))
       nil)))
  (let ((status  (or (cdr-safe (assoc "Status" info)) ""))
        (package (or (cdr-safe (assoc "Package" info)) "")))
    (cond
     ((null info)
      (message "TinyDebian: no INFO available to send a bug report."))
     ((string-match "not-installed" status)
      (message "TinyDebian: bug report skipped. %s' status is [%s]"
               package status))
     (t
      (let* ((name   (format "*mail* Debian Bug %s" package))
             buffer)
        (cond
         ((and (setq buffer (get-buffer name))
               (null (y-or-n-p
                      "[TinyDebian] delete previous bug report? ")))
          (pop-to-buffer buffer))
         (t
          (pop-to-buffer (get-buffer-create name))
          (erase-buffer)
          (let ((subject (read-string "[TinyDebian] bug Subject: ")))
            (mail-setup
             "submit@bugs.debian.org" subject nil nil nil nil))
          (message-mode)
          (tinydebian-bug-report-mail-insert-details info))))))))

;;}}}
;;{{{ Admin functions: MAIL reports

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-audit-report-tiger-make-chmod (file line)
  "Make suotable chmod command for FILE according to LINE report."
  (let* ((operand "+")
         group
         group-cmd
         type
         type-cmd)
    (when (string-match
           "should .*+have +\\([^ \t\r\n]+\\) +\\([^ \t\r\n.]+\\)"
           line)
      (setq group (match-string 1 line)
            type  (match-string 2 line))
      (if (string-match "should not" line)
          (setq operand "-"))
      (cond
       ((string= group "group")
        (setq group-cmd "g"))
       ((string= group "world")
        (setq group-cmd "o")))
      (cond
       ((string-match type "read")
        (setq type-cmd "r"))
       ((string-match type "write")
        (setq type-cmd "w"))
       ((string-match type "exec")
        (setq type-cmd "x")))
      (when (and operand type-cmd group-cmd)
        (format "chmod %s%s%s %s;" group-cmd operand type-cmd file)))))

;;; ----------------------------------------------------------------------
;;;
(defun tinydebian-audit-report-tiger (beg end)
  "Process tiger(1) mail system report on region BEG END.
The body of mail looks like:

    # Performing check of system file permissions...
    OLD: --WARN-- [perm001w] /var/log/wtmp should not have group write.
    OLD: --WARN-- [perm001w] /var/run/utmp should not have group write.
    OLD: --WARN-- [perm001w] /var/log/XFree86.0.log should not have world read.

For which a corresponding command to correct the error is generated.

    chmod g-w /var/log/wtmp;
    chmod g-w /var/run/utmp;
    chmod o-r /var/log/XFree86.0.log;

You can select region and these commands to shell `sh' with command
`shell-command-on-region' which can be called with \\[shell-command-on-region]."
  (interactive "r")
  (let* ((buffer (get-buffer-create tinydebian-:buffer-tiger))
         done
         file
         str)
    (goto-char beg)
    (while (re-search-forward
            "--WARN-- +[^ \t\r\n]+ +\\(\\([^ \t\r\n]+\\).*\\)"
            nil end)
      (setq file (match-string 2)
            str  (match-string 1))

      (unless done                      ;Draw one empty line between calls
        (setq done t)
        (ti::append-to-buffer buffer "\n"))

      (when (setq str (tinydebian-audit-report-tiger-make-chmod file str))
        (ti::append-to-buffer buffer (concat str "\n"))))

    (cond
     ((ti::buffer-empty-p buffer)
      (message "TinyDebian: Hm, region did not have --WARN-- chmod candidates."))
     (t
      (display-buffer buffer)
      (message
       (substitute-command-keys
        (concat
         "TinyDebian: [tiger] "
         "Select region and send commands to `sh' with \\[shell-command-on-region]")))))))




;;}}}

(tinydebian-install-severity-functions) ;; Auto-created functions

(provide   'tinydebian)
(run-hooks 'tinydebian-:load-hook)

;;; tinydebian.el ends here

